@@ -42,7 +42,7 @@ VERSION = "3.2"
 #   * Doc/library/stdtypes.rst, and
 #   * Doc/library/unicodedata.rst
 #   * Doc/reference/lexical_analysis.rst (two occurrences)
-UNIDATA_VERSION = "6.3.0"
+UNIDATA_VERSION = "8.0.0"
 UNICODE_DATA = "UnicodeData%s.txt"
 COMPOSITION_EXCLUSIONS = "CompositionExclusions%s.txt"
 EASTASIAN_WIDTH = "EastAsianWidth%s.txt"
@@ -99,10 +99,11 @@ EXTENDED_CASE_MASK = 0x4000
 # these ranges need to match unicodedata.c:is_unified_ideograph
 cjk_ranges = [
     ('3400', '4DB5'),
-    ('4E00', '9FCC'),
+    ('4E00', '9FD5'),
     ('20000', '2A6D6'),
     ('2A700', '2B734'),
-    ('2B740', '2B81D')
+    ('2B740', '2B81D'),
+    ('2B820', '2CEA1'),
 ]
 
 def maketables(trace=0):
